@@ -5,6 +5,7 @@ from arcor2 import DynamicParamTuple as DPT
 from arcor2 import rest
 from arcor2.data.common import ActionMetadata, Joint, Pose, StrEnum
 from arcor2.data.robot import RobotType
+from arcor2.exceptions import Arcor2NotImplemented
 from arcor2.object_types.abstract import Robot, RobotException, Settings
 
 # TODO jogging
@@ -35,7 +36,17 @@ class AbstractDobot(Robot):
     def __init__(self, obj_id: str, name: str, pose: Pose, settings: DobotSettings) -> None:
         super(AbstractDobot, self).__init__(obj_id, name, pose, settings)
 
+    def _started(self) -> bool:
+        return rest.call(rest.Method.GET, f"{self.settings.url}/started", return_type=bool)
+
+    def _stop(self) -> None:
+        rest.call(rest.Method.PUT, f"{self.settings.url}/stop")
+
     def _start(self, model: str) -> None:
+
+        if self._started():
+            self._stop()
+
         rest.call(
             rest.Method.PUT,
             f"{self.settings.url}/start",
@@ -48,7 +59,7 @@ class AbstractDobot(Robot):
         return cast(DobotSettings, super(AbstractDobot, self).settings)
 
     def cleanup(self):
-        rest.call(rest.Method.PUT, f"{self.settings.url}/stop")
+        self._stop()
 
     def get_end_effectors_ids(self) -> Set[str]:
         return {"default"}
@@ -62,10 +73,14 @@ class AbstractDobot(Robot):
     def get_end_effector_pose(self, end_effector_id: str) -> Pose:
         return rest.call(rest.Method.GET, f"{self.settings.url}/eef/pose", return_type=Pose)
 
-    def move_to_pose(self, end_effector_id: str, target_pose: Pose, speed: float) -> None:
+    def move_to_pose(self, end_effector_id: str, target_pose: Pose, speed: float, safe: bool = False) -> None:
+        if safe:
+            raise Arcor2NotImplemented("Dobot does not support safe moves.")
         self.move(target_pose, MoveType.LINEAR, speed * 100)
 
-    def move_to_joints(self, target_joints: List[Joint], speed: float) -> None:
+    def move_to_joints(self, target_joints: List[Joint], speed: float, safe: bool = False) -> None:
+        if safe:
+            raise Arcor2NotImplemented("Dobot does not support safe moves.")
         self.move(self.forward_kinematics("", target_joints), MoveType.LINEAR, speed * 100)
 
     def home(self, *, an: Optional[str] = None) -> None:
@@ -108,6 +123,38 @@ class AbstractDobot(Robot):
     def release(self, *, an: Optional[str] = None) -> None:
         rest.call(rest.Method.PUT, f"{self.settings.url}/release")
 
+    def pick(self, pick_pose: Pose, vertical_offset: float = 0.05, *, an: Optional[str] = None) -> None:
+        """Picks an item from given pose.
+
+        :param pick_pose:
+        :param vertical_offset:
+        :return:
+        """
+
+        pick_pose.position.z += vertical_offset
+        self.move(pick_pose, MoveType.JOINTS)  # pre-pick pose
+        pick_pose.position.z -= vertical_offset
+        self.move(pick_pose, MoveType.JOINTS)  # pick pose
+        self.suck()
+        pick_pose.position.z += vertical_offset
+        self.move(pick_pose, MoveType.JOINTS)  # pre-pick pose
+
+    def place(self, place_pose: Pose, vertical_offset: float = 0.05, *, an: Optional[str] = None) -> None:
+        """Places an item to a given pose.
+
+        :param place_pose:
+        :param vertical_offset:
+        :return:
+        """
+
+        place_pose.position.z += vertical_offset
+        self.move(place_pose, MoveType.JOINTS)  # pre-place pose
+        place_pose.position.z -= vertical_offset
+        self.move(place_pose, MoveType.JOINTS)  # place pose
+        self.release()
+        place_pose.position.z += vertical_offset
+        self.move(place_pose, MoveType.JOINTS)  # pre-place pose
+
     def robot_joints(self) -> List[Joint]:
         return rest.call(rest.Method.GET, f"{self.settings.url}/joints", list_return_type=Joint)
 
@@ -143,6 +190,8 @@ class AbstractDobot(Robot):
     move.__action__ = ActionMetadata(blocking=True)  # type: ignore
     suck.__action__ = ActionMetadata(blocking=True)  # type: ignore
     release.__action__ = ActionMetadata(blocking=True)  # type: ignore
+    pick.__action__ = ActionMetadata(blocking=True, composite=True)  # type: ignore
+    place.__action__ = ActionMetadata(blocking=True, composite=True)  # type: ignore
 
 
 AbstractDobot.DYNAMIC_PARAMS = {
